//CNC router
//
//Code to run on the command computer see:
//commands-plotter.c
//
//Source & info: www.HomoFaciens.de


//#include <avr/io.h>
//#include <util/delay.h>
#include <Servo.h>

//Change values below to adapt your motor

#define STEP_MARGIN_X 1L       //10 - 1000 (2)
#define MIN_DUTYCYCLE_X 80    //0 - 255 (125)
#define MAX_DUTYCYCLE_X 120    //0 - 255 (255)

#define STEP_MARGIN_Y 1L       //10 - 1000 (2)
#define MIN_DUTYCYCLE_Y 120    //0 - 255 (125)
#define MAX_DUTYCYCLE_Y 255    //0 - 255 (255)

#define STEP_MARGIN_Z 1L       //10 - 1000 (2)
#define MIN_DUTYCYCLE_Z 120    //0 - 255 (125)
#define MAX_DUTYCYCLE_Z 255    //0 - 255 (255)

#define MX_S1 A0             //Sensor input 1
#define MX_S2 A1             //Sensor input 2
#define MX_DIR 2             //Direction output motor
#define MX_PWM 3             //PWM output motor

#define MY_S1 A2             //Sensor input 1
#define MY_S2 A3             //Sensor input 2
#define MY_DIR 4             //Direction output motor
#define MY_PWM 5             //PWM output motor

#define MZ_S1 A4             //Sensor input 1
#define MZ_S2 A5             //Sensor input 2
#define MZ_DIR 7             //Direction output motor
#define MZ_PWM 6             //PWM output motor

#define END_SWITCH_X   8
#define END_SWITCH_Y  12
#define END_SWITCH_Z  13

#define RELAY    9           //Relay used to turn on milling machine
#define COOLANT 10           //Coolant pump (PWM)
#define COOLANT_PWM  70      //My pump works fine with 3.3V (255 / 12 * 3 = 70)

//Number of bytes processed before confirming USB communication with 'r'
//Must be MAX_ARDUINO_BUFFER in commands-CNC.c
#define MAX_ARDUINO_BUFFER 10

int MxSensorRead2 = 0;
int MxSensorRead1 = 0;
int MxDutyCycle = 100; // 10 - 255
unsigned long MxSetPoint = 0;
unsigned long MxActualPoint = 0;
byte MxStep = 0;
byte MxStepDone = 0;

int MySensorRead2 = 0;
int MySensorRead1 = 0;
int MyDutyCycle = 100; // 10 - 255
unsigned long MySetPoint = 0;
unsigned long MyActualPoint = 0;
byte MyStep = 0;
byte MyStepDone = 0;

int MzSensorRead2 = 0;
int MzSensorRead1 = 0;
int MzDutyCycle = 100; // 10 - 255
unsigned long MzSetPoint = 0;
unsigned long MzActualPoint = 0;
byte MzStep = 0;
byte MzStepDone = 0;

byte readByte = 0;
byte readByteNext = 0;

byte bufferSize = 0;


unsigned long stepTime = 1;

unsigned long millisStart = 0;


void establishContact() {
  while (Serial.available() <= 0) {
    Serial.print('X');   // send a capital X
    delay(300);
  }
}

void printValues() {
  Serial.print(MxActualPoint);
  Serial.print(", ");
  Serial.print(MxSetPoint);
  Serial.print(", ");
  Serial.println(MxStep);
}

void setup(){

  pinMode(RELAY, OUTPUT);
  digitalWrite(RELAY, 0);
  pinMode(COOLANT, OUTPUT);
  analogWrite(COOLANT, 0);


  pinMode(MX_DIR, OUTPUT);
  pinMode(MX_PWM, OUTPUT);
  analogWrite(MX_PWM, 0);
  digitalWrite(MX_DIR, 0);

  pinMode(MX_S1, INPUT_PULLUP);
  pinMode(MX_S2, INPUT_PULLUP);


  pinMode(MY_DIR, OUTPUT);
  pinMode(MY_PWM, OUTPUT);
  analogWrite(MY_PWM, 0);
  digitalWrite(MY_DIR, 0);

  pinMode(MY_S1, INPUT);
  pinMode(MY_S2, INPUT);


  pinMode(MZ_DIR, OUTPUT);
  pinMode(MZ_PWM, OUTPUT);
  analogWrite(MZ_PWM, 0);
  digitalWrite(MZ_DIR, 0);

  pinMode(MZ_S1, INPUT);
  pinMode(MZ_S2, INPUT);

  pinMode(END_SWITCH_X, INPUT);
  pinMode(END_SWITCH_Y, INPUT);
  pinMode(END_SWITCH_Z, INPUT);

  stepTime = 1;
  readByte = 0;
  readByteNext = 0;

  MxStep = 0;
  MyStep = 0;
  MzStep = 0;

  MxStepDone = 1;
  MyStepDone = 1;
  MzStepDone = 1;

  MxSetPoint     = 300000;
  MxActualPoint  = 300000;

  MySetPoint     = 300000;
  MyActualPoint  = 300000;

  MzSetPoint     = 300000;
  MzActualPoint  = 300000;

  bufferSize = 0;

  // start serial port at 115200 bps:
  Serial.begin(115200);
  establishContact();  // send a byte to establish contact until receiver responds
}


void loop(){

//  stepTime = 0;// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  millisStart = millis();
    do{
      MzSensorRead1 = digitalRead(MZ_S1);
      MzSensorRead2 = digitalRead(MZ_S2);

      if(MzSensorRead2 == 0 && MzSensorRead1 == 0){
        if(MzStep == 3){
          MzActualPoint++;
        }
        if(MzStep == 1){
          MzActualPoint--;
        }
        MzStep = 0;
      }

      if(MzSensorRead2 == 1 && MzSensorRead1 == 0){
        if(MzStep == 0){
          MzActualPoint++;
        }
        if(MzStep == 2){
          MzActualPoint--;
        }
        MzStep = 1;
      }

      if(MzSensorRead2 == 1 && MzSensorRead1 == 1){
        if(MzStep == 1){
          MzActualPoint++;
        }
        if(MzStep == 3){
          MzActualPoint--;
        }
        MzStep = 2;
      }

      if(MzSensorRead2 == 0 && MzSensorRead1 == 1){
        if(MzStep == 2){
          MzActualPoint++;
        }
        if(MzStep == 0){
          MzActualPoint--;
        }
        MzStep = 3;
      }

      if(abs(MzSetPoint - MzActualPoint) < STEP_MARGIN_Z){
        analogWrite(MZ_PWM, 0);
        digitalWrite(MZ_DIR, 0);
        MzStepDone = 1;
        MzDutyCycle = MIN_DUTYCYCLE_Z;
      }
      else{
        if(MzActualPoint < MzSetPoint){
          digitalWrite(MZ_DIR, 1);
          analogWrite(MZ_PWM, 255 - MzDutyCycle);
        }
        if(MzActualPoint > MzSetPoint){
          digitalWrite(MZ_DIR, 0);
          analogWrite(MZ_PWM, MzDutyCycle);
        }
      }

      MySensorRead1 = digitalRead(MY_S1);
      MySensorRead2 = digitalRead(MY_S2);

      if(MySensorRead2 == 0 && MySensorRead1 == 0){
        if(MyStep == 3){
          MyActualPoint++;
        }
        if(MyStep == 1){
          MyActualPoint--;
        }
        MyStep = 0;
      }

      if(MySensorRead2 == 1 && MySensorRead1 == 0){
        if(MyStep == 0){
          MyActualPoint++;
        }
        if(MyStep == 2){
          MyActualPoint--;
        }
        MyStep = 1;
      }

      if(MySensorRead2 == 1 && MySensorRead1 == 1){
        if(MyStep == 1){
          MyActualPoint++;
        }
        if(MyStep == 3){
          MyActualPoint--;
        }
        MyStep = 2;
      }

      if(MySensorRead2 == 0 && MySensorRead1 == 1){
        if(MyStep == 2){
          MyActualPoint++;
        }
        if(MyStep == 0){
          MyActualPoint--;
        }
        MyStep = 3;
      }

      if(abs(MySetPoint - MyActualPoint) < STEP_MARGIN_Y){
        analogWrite(MY_PWM, 0);
        digitalWrite(MY_DIR, 0);
        MyStepDone = 1;
        MyDutyCycle = MIN_DUTYCYCLE_Y;
      }
      else{
        if(MyActualPoint < MySetPoint){
          digitalWrite(MY_DIR, 1);
          analogWrite(MY_PWM, 255 - MyDutyCycle);
        }
        if(MyActualPoint > MySetPoint){
          digitalWrite(MY_DIR, 0);
          analogWrite(MY_PWM, MyDutyCycle);
        }
      }



      MxSensorRead1 = digitalRead(MX_S1);
      MxSensorRead2 = digitalRead(MX_S2);

      if(MxSensorRead2 == 0 && MxSensorRead1 == 0){
        if(MxStep == 3){
          MxActualPoint++;
//          printValues();
        }
        if(MxStep == 1){
          MxActualPoint--;
//          printValues();
        }
        MxStep = 0;
      }

      if(MxSensorRead2 == 1 && MxSensorRead1 == 0){
        if(MxStep == 0){
          MxActualPoint++;
//          printValues();
        }
        if(MxStep == 2){
          MxActualPoint--;
//          printValues();
        }
        MxStep = 1;
      }

      if(MxSensorRead2 == 1 && MxSensorRead1 == 1){
        if(MxStep == 1){
          MxActualPoint++;
//          printValues();
        }
        if(MxStep == 3){
          MxActualPoint--;
//          printValues();
        }
        MxStep = 2;
      }

      if(MxSensorRead2 == 0 && MxSensorRead1 == 1){
        if(MxStep == 2){
          MxActualPoint++;
//          printValues();
        }
        if(MxStep == 0){
          MxActualPoint--;
//          printValues();
        }
        MxStep = 3;
      }

      if(abs(MxSetPoint - MxActualPoint) < STEP_MARGIN_X){
        analogWrite(MX_PWM, 0);
        digitalWrite(MX_DIR, 0);
        MxStepDone = 1;
        MxDutyCycle = MIN_DUTYCYCLE_X;
      }
      else{
        if(MxActualPoint < MxSetPoint){
          digitalWrite(MX_DIR, 1);
          analogWrite(MX_PWM, 255 - MxDutyCycle);
        }
        if(MxActualPoint > MxSetPoint){
          digitalWrite(MX_DIR, 0);
          analogWrite(MX_PWM, MxDutyCycle);
        }
      }


      if(readByteNext == 0){
        if (Serial.available() > 0){//if we get a valid byte, read from serial:
          //get incoming byte:
          readByteNext = Serial.read();
      //    bufferSize++;
      //    if(bufferSize == MAX_ARDUINO_BUFFER){
            Serial.write('r');   // send a 'r' to initiate next data from computer
      //      bufferSize = 0;
      //    }
        }//if (Serial.available() > 0)
      }


    }while((MzStepDone == 0 || MyStepDone == 0 || MxStepDone == 0) || millis() - millisStart < stepTime);


  if(readByteNext != 0 && readByte == 0){
    readByte = readByteNext;
    readByteNext = 0;
  }

  if(readByte != 0){

    if(readByte == 'a'){
      MxSetPoint++;
      MxDutyCycle = MAX_DUTYCYCLE_X;
      MxStepDone = 0;
    }
    if(readByte == 'b'){
      MxSetPoint--;
      MxDutyCycle = MAX_DUTYCYCLE_X;
      MxStepDone = 0;
    }

    if(readByte == 'A'){
      MxSetPoint+=1200;
      MxDutyCycle = MAX_DUTYCYCLE_X;
      MxStepDone = 0;
    }
    if(readByte == 'B'){
      MxSetPoint-=1200;
      MxDutyCycle = MAX_DUTYCYCLE_X;
      MxStepDone = 0;
    }

    if(readByte == 'c'){
      MySetPoint++;
      MyDutyCycle = MAX_DUTYCYCLE_Y;
      MyStepDone = 0;
    }
    if(readByte == 'd'){
      MySetPoint--;
      MyDutyCycle = MAX_DUTYCYCLE_Y;
      MyStepDone = 0;
    }

    if(readByte == 'e'){
      MzSetPoint++;
      MzDutyCycle = MAX_DUTYCYCLE_Z;
      MzStepDone = 0;
    }
    if(readByte == 'f'){
      MzSetPoint--;
      MzDutyCycle = MAX_DUTYCYCLE_Z;
      MzStepDone = 0;
    }

    if(readByte == 'g'){
      MxSetPoint+=1200;
      MxDutyCycle = MAX_DUTYCYCLE_X;
      MxStepDone = 0;

      MySetPoint+=1200;
      MyDutyCycle = MAX_DUTYCYCLE_Y;
      MyStepDone = 0;
    }

    if(readByte == 'h'){
      MxSetPoint-=1200;
      MxDutyCycle = MAX_DUTYCYCLE_X;
      MxStepDone = 0;

      MySetPoint-=1200;
      MyDutyCycle = MAX_DUTYCYCLE_Y;
      MyStepDone = 0;
    }


    if(readByte == 'P'){
      analogWrite(COOLANT, COOLANT_PWM);
      readByte = 0;
    }
    if(readByte == 'p'){
      analogWrite(COOLANT, 0);
      readByte = 0;
    }

    //Request switch state
    if(readByte == 'Q'){
      if(digitalRead(END_SWITCH_X)==1){
        Serial.print('X');
      }
      else{
        Serial.print('x');
      }

      if(digitalRead(END_SWITCH_Y)==1){
        Serial.print('Y');
      }
      else{
        Serial.print('y');
      }

      if(digitalRead(END_SWITCH_Z)==1){
        Serial.print('Z');
      }
      else{
        Serial.print('z');
      }
    }

    if(readByte == 'R'){
      digitalWrite(RELAY, 1);
      readByte = 0;
    }
    if(readByte == 'r'){
      digitalWrite(RELAY, 0);
      readByte = 0;
    }

    if(readByte == 'S'){
      if(stepTime < 1000){
        stepTime+=1;
      }
    }
    if(readByte == 's'){
      if(stepTime > 0){
        stepTime-=1;
      }
    }

    readByte = readByteNext;
    readByteNext = 0;
  }//if(readByte > 0){

}


